package com.io.yutian.auchestshop.manager;

import com.io.yutian.auchestshop.AuChestShop;
import com.io.yutian.auchestshop.shop.Shop;
import com.io.yutian.auchestshop.shop.ShopAction;
import com.io.yutian.auchestshop.shop.ShopInteractiveInfo;
import com.io.yutian.auchestshop.util.TradeHelper;
import com.io.yutian.aulib.lang.Lang;
import com.io.yutian.aulib.util.StringUtil;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class InteractiveManager {

    private final Map<UUID, ShopInteractiveInfo> actions = new HashMap<>();

    public void handler(Player player, String message) {
        ShopInteractiveInfo info = actions.remove(player.getUniqueId());
        if (message.equalsIgnoreCase("cancel") || message.equalsIgnoreCase("取消")) {
            player.sendMessage(Lang.get("auchestshop.action.cancel"));
            return;
        }
        ShopAction action = info.getAction();
        String arg = message.trim();
        Shop shop = info.getShop();
        if (AuChestShop.getShopManager().getShop(shop.getUUID()) == null) {
            player.sendMessage(Lang.get("auchestshop.action.cancel"));
            return;
        }
        if (action == ShopAction.CREATE) {
            if (!StringUtil.isDouble(arg)) {
                player.sendMessage(Lang.get("auchestshop.action.error-1"));
                return;
            }
            double i = Double.parseDouble(arg);
            if (i <= 0) {
                player.sendMessage(Lang.get("auchestshop.action.error-6"));
                return;
            }
            int length = arg.contains(".") ? arg.length() - arg.indexOf(".") + 1 : 0;
            if (length >= 3) {
                player.sendMessage(Lang.get("auchestshop.action.error-3"));
                return;
            }
            if (i >= Double.MAX_VALUE) {
                player.sendMessage(Lang.get("auchestshop.action.error-4"));
                return;
            }
            ItemStack itemStack = info.getItemStack();
            shop.setItem(itemStack);
            shop.setPrice(i);
            AuChestShop.getSQLIO().saveShop(shop);
            player.sendMessage(Lang.get("auchestshop.action.success-create"));
        } else if (action == ShopAction.PURCHASE) {
            boolean all = arg.equalsIgnoreCase("all");
            int i = 0;
            if (all) {
                i = -1;
            } else {
                if (!StringUtil.isInt(arg)) {
                    player.sendMessage(Lang.get("auchestshop.action.error-1"));
                    return;
                }
                i = Integer.parseInt(arg);
                if (i <= 0) {
                    player.sendMessage(Lang.get("auchestshop.action.error-2"));
                    return;
                }
                if (i >= Integer.MAX_VALUE) {
                    player.sendMessage(Lang.get("auchestshop.action.error-4"));
                    return;
                }
            }
            TradeHelper.trade(player, shop, i);
        }
    }

    public ShopInteractiveInfo put(UUID uuid, ShopInteractiveInfo info) {
        return this.actions.put(uuid, info);
    }

    public ShopInteractiveInfo remove(UUID uuid) {
        return this.actions.remove(uuid);
    }

    public ShopInteractiveInfo get(UUID uuid) {
        return this.actions.get(uuid);
    }

    public boolean containsKey(UUID uuid) {
        return this.actions.containsKey(uuid);
    }

    public Map<UUID, ShopInteractiveInfo> getActions() {
        return actions;
    }

}
