package com.io.yutian.auchestshop.manager;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.io.yutian.auchestshop.AuChestShop;
import com.io.yutian.auchestshop.database.SQLIO;
import com.io.yutian.auchestshop.listener.ShopListener;
import com.io.yutian.auchestshop.shop.*;
import com.io.yutian.auchestshop.util.ItemStackUtil;
import com.io.yutian.auchestshop.util.ShopUtil;
import com.io.yutian.aulib.lang.Lang;
import org.apache.commons.lang.StringUtils;
import org.bukkit.Chunk;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.data.type.Chest;
import org.bukkit.entity.Player;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class ShopManager {

    private final Map<String, Map<ShopChunk, Map<ShopLocation, Shop>>> shops = Maps.newConcurrentMap();
    private final Set<Shop> loadedShops = Sets.newConcurrentHashSet();

    public void loadAll(SQLIO sqlio) {
        List<Shop> shopList = sqlio.loadAllShops();
        shopList.forEach(this::addShop);
    }

    public void saveAll(SQLIO sqlio) {
        sqlio.saveAllShops(getAllShops());
    }

    public Shop createShop(Player player, Block block, ShopType shopType) {
        Location location = block.getLocation();
        ShopLocation shopLocation = ShopLocation.fromLocation(location);
        Shop shop = new Shop(player.getUniqueId(), shopLocation, shopType);
        addShop(shop);
        AuChestShop.getSQLIO().saveShop(shop);
        Block block1 = block.getRelative(BlockFace.UP, 1);
        block1.setType(block.getType());
        Chest chest = (Chest) block1.getState().getBlockData();
        chest.setDoubleBlockHalf(Chest.ChestDoubleBlockHalfType.UPPER);
        chest.setShapeLevel(12);
        block1.setBlockData(chest);
        Chest chest1 = (Chest) block.getState().getBlockData();
        chest1.setDoubleBlockHalf(Chest.ChestDoubleBlockHalfType.LOWER);
        chest1.setShapeLevel(12);
        block.setBlockData(chest1);
        return shop;
    }

    public void handleInteract(Player player, PlayerInteractEvent event, Shop shop, ShopListener.ClickType clickType) {
        Action action = event.getAction();
        if (action == Action.LEFT_CLICK_BLOCK) {
            if (shop.getItem() == null) {
                if (event.getItem() != null && !event.getItem().getType().equals(Material.AIR)) {
                    ShopInteractiveInfo interactiveInfo = new ShopInteractiveInfo(event.getItem().clone(), shop.getLocation(), shop, ShopAction.CREATE, clickType, System.currentTimeMillis());
                    ItemStack item = event.getItem().clone();
                    player.sendMessage(Lang.get("auchestshop.action.message-1", ItemStackUtil.getItemName(item)));
                    AuChestShop.getInteractiveManager().put(player.getUniqueId(), interactiveInfo);
                    return;
                }
                return;
            }
            ItemStack item = shop.getItem();
            Inventory inventory = shop.getInventory();
            int i =  shop.getShopType() == ShopType.BUYING ? ItemStackUtil.countItems(player.getInventory(), item) : ItemStackUtil.countItems(inventory, item);
            player.sendMessage(Lang.get("auchestshop.action.message-" + (shop.getShopType() == ShopType.BUYING ? 2 : 3), ItemStackUtil.getItemName(item), i));
            ShopInteractiveInfo interactiveInfo = new ShopInteractiveInfo(null, shop.getLocation(), shop, ShopAction.PURCHASE, clickType, System.currentTimeMillis());
            AuChestShop.getInteractiveManager().put(player.getUniqueId(), interactiveInfo);
        } else if (action == Action.RIGHT_CLICK_BLOCK) {
            if (player.isSneaking() && shop.getOwner().equals(player.getUniqueId())) {
                return;
            }
            ItemStack item = shop.getItem();
            Inventory inventory = shop.getInventory();
            int i =  shop.getShopType() == ShopType.BUYING ? ItemStackUtil.countItems(player.getInventory(), item) : ItemStackUtil.countItems(inventory, item);
            player.sendMessage(Lang.get("auchestshop.action.message-" + (shop.getShopType() == ShopType.BUYING ? 2 : 3), ItemStackUtil.getItemName(item), i));
            ShopInteractiveInfo interactiveInfo = new ShopInteractiveInfo(null, shop.getLocation(), shop, ShopAction.PURCHASE, clickType, System.currentTimeMillis());
            AuChestShop.getInteractiveManager().put(player.getUniqueId(), interactiveInfo);
            event.setCancelled(true);
        }
    }

    public @NotNull List<Shop> getAllShops() {
        List<Shop> shopsCollected = new ArrayList<>();
        for (Map<ShopChunk, Map<ShopLocation, Shop>> shopMapData : getShops().values()) {
            for (Map<ShopLocation, Shop> shopData : shopMapData.values()) {
                shopsCollected.addAll(shopData.values());
            }
        }
        return shopsCollected;
    }

    public @NotNull List<Shop> getAllShops(@NotNull UUID playerUUID) {
        List<Shop> playerShops = new ArrayList<>();
        for (Shop shop : getAllShops()) {
            UUID shopUuid = shop.getOwner();
            if (playerUUID.equals(shopUuid)) {
                playerShops.add(shop);
            }
        }
        return playerShops;
    }


    public @Nullable Shop getShop(UUID shopId) {
        for (Shop shop : getAllShops()) {
            if (shop.getUUID().equals(shopId)) {
                return shop;
            }
        }
        return null;
    }

    public @Nullable Shop getShop(@NotNull Location location) {
        if (!ShopUtil.isShoppables(location.getBlock().getType())) {
            return null;
        }
        ShopChunk shopChunk = ShopChunk.fromLocation(location);
        final Map<ShopLocation, Shop> inChunk = getShops(shopChunk);
        if (inChunk == null) {
            return null;
        }
        ShopLocation shopLocation = ShopLocation.fromLocation(location);
        return inChunk.get(shopLocation);
    }

    public Map<String, Map<ShopChunk, Map<ShopLocation, Shop>>> getShops() {
        return this.shops;
    }

    public Map<ShopLocation, Shop> getShops(@NotNull Chunk chunk) {
        return getShops(chunk.getWorld().getName(), chunk.getX(), chunk.getZ());
    }

    public Map<ShopLocation, Shop> getShops(@NotNull String world, int chunkX, int chunkZ) {
        final Map<ShopChunk, Map<ShopLocation, Shop>> inWorld = this.getShops(world);
        if (inWorld == null) {
            return null;
        }
        return inWorld.get(new ShopChunk(world, chunkX, chunkZ));
    }

    public Map<ShopLocation, Shop> getShops(@NotNull ShopChunk shopChunk) {
        return getShops(shopChunk.getWorld(), shopChunk.getX(), shopChunk.getZ());
    }

    public Map<ShopChunk, Map<ShopLocation, Shop>> getShops(@NotNull String world) {
        return this.shops.get(world);
    }

    public @NotNull List<Shop> getShopsInWorld(@NotNull World world) {
        final List<Shop> worldShops = new ArrayList<>();
        for (final Shop shop : getAllShops()) {
            ShopLocation location = shop.getLocation();
            if (Objects.equals(location.getWorld(), world)) {
                worldShops.add(shop);
            }
        }
        return worldShops;
    }

    public @NotNull List<Shop> getShopsInWorld(@NotNull String worldName) {
        final List<Shop> worldShops = new ArrayList<>();
        for (final Shop shop : getAllShops()) {
            ShopLocation location = shop.getLocation();
            if (StringUtils.equals(worldName, location.getWorld())) {
                worldShops.add(shop);
            }
        }
        return worldShops;
    }

    public void addShop(@NotNull Shop shop) {
        ShopLocation shopLocation = shop.getLocation();
        String world = shopLocation.getWorld();
        Map<ShopChunk, Map<ShopLocation, Shop>> map = shops.computeIfAbsent(world, k -> new HashMap<>());
        int x = (int) Math.floor((shopLocation.getX()) / 16.0);
        int z = (int) Math.floor((shopLocation.getZ()) / 16.0);
        ShopChunk shopChunk = new ShopChunk(world, x, z);
        Map<ShopLocation, Shop> inChunk = map.computeIfAbsent(shopChunk, k -> new HashMap<>());
        inChunk.put(shopLocation, shop);
        loadedShops.add(shop);
    }

    public void removeShop(@NotNull Shop shop) {
        ShopLocation location = shop.getLocation();
        ShopChunk shopChunk = ShopChunk.fromLocation(location);
        final Map<ShopLocation, Shop> inChunk = getShops(shopChunk);
        if (inChunk != null) {
            inChunk.remove(location);
        }
        this.loadedShops.remove(shop);
        AuChestShop.getSQLIO().deleteShop(shop);
    }

    public Set<Shop> getLoadedShops() {
        return loadedShops;
    }

}
