package com.io.yutian.auchestshop.shop;

import com.io.yutian.auchestshop.shop.log.ShopLog;
import com.io.yutian.aulib.serialize.SerializeIgnore;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.Chest;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;

import java.util.Objects;
import java.util.UUID;

public class Shop {

    private final UUID uuid;
    private final ShopLocation location;
    private final UUID owner;
    private double price;
    private ItemStack item;
    private ShopType shopType;
    private boolean unlimited;
    private ShopLog shopLog;

    @SerializeIgnore
    protected Location chestLocation;
    @SerializeIgnore
    protected Block chestBlock;

    public Shop(UUID owner, ShopLocation location, ShopType shopType) {
        this(UUID.randomUUID(), owner, location, null, 0, shopType, false, new ShopLog());
    }

    public Shop(UUID uuid, UUID owner, ShopLocation location, ItemStack item, double price, ShopType shopType, boolean unlimited, ShopLog shopLog) {
        this.uuid = uuid;
        this.owner = owner;
        this.location = location;
        this.item = item;
        this.price = price;
        this.shopType = shopType;
        this.unlimited = unlimited;
        this.shopLog = shopLog;
        updateChest();
    }

    public void updateChest() {
        World world = Bukkit.getWorld(location.getWorld());
        this.chestLocation = new Location(world, location.getX(), location.getY(), location.getZ());
        this.chestBlock = this.chestLocation.getBlock();
    }

    public Inventory getInventory() {
        if (chestLocation == null) {
            updateChest();
        }
        Chest chest = (Chest) chestBlock.getState();
        return chest.getInventory();
    }

    public int getInventoryCount() {
        return 0;
    }

    public void setItem(ItemStack item) {
        this.item = item;
    }

    public void setPrice(double price) {
        this.price = price;
    }

    public UUID getUUID() {
        return uuid;
    }

    public ShopLocation getLocation() {
        return location;
    }

    public UUID getOwner() {
        return owner;
    }

    public double getPrice() {
        return price;
    }

    public ShopType getShopType() {
        return shopType;
    }

    public boolean isUnlimited() {
        return unlimited;
    }

    public ItemStack getItem() {
        return item;
    }

    public ShopLog getShopLog() {
        return shopLog;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Shop shop = (Shop) o;
        return Objects.equals(uuid, shop.uuid) && Objects.equals(getOwner(), shop.getOwner());
    }

    @Override
    public int hashCode() {
        int result = Objects.hashCode(uuid);
        result = 31 * result + Objects.hashCode(getOwner());
        return result;
    }
}
